-----------------------------------------------------------------------------

AloftTag = { }
AloftTagMethods = { }
AloftTagDirectAccessData = { isBoss = true, level = true, name = true, health = true, maxHealth = true, healthBarValue = true, healthBarMaxValue = true }
AloftTagData = 
{ 
	health 			= { events = "Aloft:OnHealthBarValueChanged"},
	healthBarValue 	= { events = "Aloft:OnHealthBarValueChanged"},
	maxHealth 		= { events = "Aloft:OnHealthBarValueChanged"},
}

-----------------------------------------------------------------------------

AloftTagMethods.angle = function(aloftData, value) return value and value ~= "" and string.format("<%s>", value) or nil end
AloftTagMethods.bracket = function(aloftData, value) return value and value ~= "" and string.format("(%s)", value) or nil end
AloftTagMethods.brace = function(aloftData, value) 	return value and value ~= "" and string.format("{%s}", value) or nil end

AloftTagMethods.healthBarDeficit = function(aloftData, value)
	return aloftData.healthBarMaxValue - aloftData.healthBarValue
end
AloftTagData.healthBarDeficit = { events = "Aloft:OnHealthBarValueChanged" }

function AloftTagMethods.healthDeficit(aloftData, value)
	if aloftData.health and aloftData.maxHealth then
		return aloftData.maxHealth - aloftData.health
	else
		return nil
	end
end
AloftTagData.healthDeficit = { events = "Aloft:OnHealthBarValueChanged" }

AloftTagMethods.healthPercent = function(aloftData, value)
	return floor(100*aloftData.healthBarValue / aloftData.healthBarMaxValue)
end
AloftTagData.healthPercent = { events = "Aloft:OnHealthBarValueChanged" }

function AloftTagMethods.hideKnownHealth(aloftData, value)
	return not aloftData.health and value or nil
end

AloftTagMethods.hideGroup = function(aloftData, value)
	return aloftData.healthBarMaxValue == 100 and value or nil
end

AloftTagMethods.hideNonGroup = function(aloftData, value)
	return aloftData.healthBarMaxValue ~= 100 and value or nil
end

AloftTagMethods.hideMaxHealth = function(aloftData, value)
	if aloftData.healthBarValue == aloftData.healthBarMaxValue then
		return nil
	else
		return value
	end
end
AloftTagData.hideMaxHealth = { events = "Aloft:OnHealthBarValueChanged" }

AloftTagMethods.hideZero = function(aloftData, value)
	if value and value ~= 0 then
		return value
	else
		return nil
	end
end

AloftTagMethods.lower = function(aloftData, value)
	if type(value) == "string" then
		return value:lower()
	end
	return value
end

AloftTagMethods.negate = function(aloftData, value)
	if type(value) == "number" then
		return -value
	end
	return value
end

AloftTagMethods["not"] = function(aloftData, value)
	return not value
end

AloftTagMethods.percent = function(aloftData, value)
	if type(value) == "number" then
		return value.."%"
	end
	return value
end

AloftTagMethods.short = function(aloftData, value)
	if type(value) == "number" then
		if value < 10000 then
			-- Force it to an integer
			return string.format("%d", value)
		elseif value < 100000 then
			return string.format("%.1fk", value / 1000)
		elseif value < 1000000 then
			return string.format("%dk", value / 1000)
		else
			return string.format("%.2fm", value / 1000000)
		end
	end
	return value
end

AloftTagMethods.upper = function(aloftData, value)
	if type(value) == "string" then
		return value:upper()
	end
	return value
end

AloftTagMethods.white = function(aloftData, value) return value and ("|r%s"):format(value) or nil end
AloftTagMethods.red = function(aloftData, value) return value and ("|cffff0000%s|r"):format(value) or nil end
AloftTagMethods.green = function(aloftData, value) return value and ("|cff00ff00%s|r"):format(value) or nil end
AloftTagMethods.blue = function(aloftData, value) return value and ("|cff0000ff%s|r"):format(value) or nil end
AloftTagMethods.cyan = function(aloftData, value) return value and ("|cff00ffff%s|r"):format(value) or nil end
AloftTagMethods.fuchsia = function(aloftData, value) return value and ("|cffff00ff%s|r"):format(value) or nil end
AloftTagMethods.yellow = function(aloftData, value) return value and ("|cffffff00%s|r"):format(value) or nil end
AloftTagMethods.gray = function(aloftData, value) return value and ("|cffafafaf%s|r"):format(value) or nil end
AloftTagMethods.difficultyColor = function(aloftData, value) return value and ("|cff%02x%02x%02x%s|r"):format(aloftData.levelTextR*255, aloftData.levelTextG*255, aloftData.levelTextB*255, value) or nil end

-----------------------------------------------------------------------------

function AloftTag:ProcessTag(tag, offset, dataTable, priorResult)

	local s,e = tag:find(":", offset)
	local quotedS,quotedE = tag:find("\"[^\"]*\"", offset)

	if quotedS and quotedS < (s or #tag) then
		if quotedE == #tag then
			if offset == 1 then
				return string.format("%s", tag:sub(quotedS, quotedE))
			else
				return string.format("%s and %s", priorResult, tag:sub(quotedS, quotedE))
			end
		elseif tag:sub(quotedE+1, quotedE+1) == ":"	then
			if offset == 1 then
				return AloftTag:ProcessTag(tag, quotedE+2, dataTable, string.format("%s", tag:sub(quotedS, quotedE)))
			else
				return AloftTag:ProcessTag(tag, quotedE+2, dataTable, string.format("%s and %s", priorResult, tag:sub(quotedS, quotedE)))
			end
		else
			Aloft:Print("Tag error near quotes")
			dataTable.error = true
			return "\"\""
		end
	else
		local identifier = tag:sub(offset,s and s-1 or nil)
		local data = AloftTagData[identifier]
		if data then
			if data.events then
				if type(data.events) == "string" then
					dataTable.events[data.events] = true
				end
			end
			if data.data then
				if type(data.data) == "string" then
					dataTable.data[data.data] = true
				end
			end
		end
		if AloftTagDirectAccessData[identifier] then
			if s then
				return AloftTag:ProcessTag(tag, e+1, dataTable, string.format("aloftData.%s", identifier))
			else
				return string.format("aloftData.%s", identifier)
			end
		elseif AloftTagMethods[identifier] then
			if s then
				if offset == 1 then
					return AloftTag:ProcessTag(tag, e+1, dataTable, string.format("AloftTagMethods[\"%s\"](aloftData)", identifier))
				else
					return AloftTag:ProcessTag(tag, e+1, dataTable, string.format("AloftTagMethods[\"%s\"](aloftData, %s)", identifier, priorResult))
				end
			else
				if offset == 1 then
					return string.format("AloftTagMethods[\"%s\"](aloftData)", tag:sub(offset))
				else
					return string.format("AloftTagMethods[\"%s\"](aloftData, %s)", tag:sub(offset), priorResult)
				end
			end
		else
			Aloft:Print("Invalid tag: "..identifier)
			dataTable.error = true
			return "\"\""
		end
	end
end


-- Returns a table that contains a method to execute, required data and events and the tag
function AloftTag:CreateTag(tag)
	if not tag then tag = "" end

	local dataTable = { data = { }, events = { }, tag = tag }
	local functionStringList = { "return function(aloftData) return " }

	local offset = 1
	repeat
		local s, e = tag:find("%[[^[]+%]", offset)
		if offset ~= 1 then
			table.insert(functionStringList, "..")
		end
		if s then
			if s ~= offset then
				table.insert(functionStringList, string.format("%q..", tag:sub(offset, s-1)))
			end
			table.insert(functionStringList, string.format([[(%s or "")]], AloftTag:ProcessTag(tag:sub(s+1, e-1), 1, dataTable, "")))
			offset = e+1
			if offset > #tag then
				offset = nil
			end
		else
			table.insert(functionStringList, string.format("%q", tag:sub(offset)))
			offset = nil
		end
	until offset == nil

	local functionString
	if dataTable.error then
		functionString = [[return function() return "Tag Error" end]]
	else
		table.insert(functionStringList, " end")
		functionString = table.concat(functionStringList)
	end
	
--[[
	AloftDebug(string.format("Tag: %s", dataTable.tag))
	for event in pairs(dataTable.events) do
		AloftDebug(string.format(" - event: %s", event))
	end
	for data in pairs(dataTable.data) do
		AloftDebug(string.format(" - data: %s", data))
	end
	AloftDebug("Compiled method: "..functionString)
--]]--
	dataTable.method = loadstring(functionString)()

	return dataTable
end

-----------------------------------------------------------------------------
