-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftHealthBar")
local SML = AceLibrary("SharedMedia-1.0")
local SML_statusbar = SML:List("statusbar")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Health Bar"] = true,
	["Health Bar options"] = true,

	["Texture"] = true,
	["Sets the health bar texture"] = true,
	["Alpha"] = true,
	["Sets the health bar alpha"] = true,

	["Size"] = true,
	["Adjust health bar's size"] = true,
	["Width"] = true,
	["Sets the width of the health bar"] = true,
	["Height"] = true,
	["Sets the height of the health bar"] = true,
	
	["Position"] = true,
	["Adjust the health bar's position"] = true,
	["Anchor"] = true, 
	["Sets the anchor for the health bar"] = true,
	["Anchor To"] = true, 
	["Sets the relative point of the frame to anchor the health bar"] = true,
	["X Offset"] = true,
	["X offset of the health bar"] = true,
	["Y Offset"] = true,
	["Y offset of the health bar"] = true,
	
	["Colors"] = true,
	["Customize colors"] = true,
	["Color by Class"] = true,
	["Sets the health bar color of players according to class"] = true,
	["Color Hostile by Class"] = true,
	["Sets the health bar color of hostile players according to class"] = true,
	["Backdrop Color"] = true,
	["Sets the backdrop color of the health bar"] = true,
	["Pet"] = true,
	["Sets the health bar color for your pet"] = true,
	["Friendly Players"] = true,
	["Sets the health bar color for friendly players"] = true,
	["Friendly Pets"] = true,
	["Sets the health bar color for friendly pets"] = true,
	["Friendly NPCs"] = true,
	["Sets the health bar color for friendly NPC Bosses"] = true,
	["Friendly Boss"] = true,
	["Sets the health bar color for friendly NPCs"] = true,
	["Hostile Players"] = true,
	["Sets the health bar color for hostile players"] = true,
	["Hostile Pets"] = true,
	["Sets the health bar color for hostile pets"] = true,
	["Hostile NPCs"] = true,
	["Sets the health bar color for hostile units"] = true,
	["Hostile Boss"] = true,
	["Sets the health bar color for hostile boss units"] = true,
	["Neutral Units"] = true,
	["Sets the health bar color for neutral units"] = true,
	["Group Pets"] = true,
	["Sets the health bar color for group pets"] = true,
	["Reset to Defaults"] = true,
	["Resets all colors to their defaults"] = true,

} end)

-----------------------------------------------------------------------------

local AloftHealthBar = Aloft:NewModule("HealthBar", "AceEvent-2.0")
AloftHealthBar.db = Aloft:AcquireDBNamespace("healthBar")
Aloft:RegisterDefaults("healthBar", "profile", {
	texture			= "Blizzard",
	alpha			= 1.0,
	colorByClass	= true,
	colorHostileByClass = true,
	width			= 115,
	height			= 11,
	offsetX			= 0,
	offsetY			= 0,
	colors =
	{
		backdropColor	= { 0.25, 0.25, 0.25, 0.5 },
		friendlyPlayer	= { 0, 0, 1 },
		friendlyPet		= { 0, 1, 0 },
		friendlyNPC		= { 0, 1, 0 },
		friendlyBoss	= { 0, 1, 0 },
		hostilePlayer	= { 1, 0, 0 },
		hostilePet		= { 1, 0, 0 },
		hostileNPC		= { 1, 0, 0 },
		hostileBoss		= { 1, 0, 0 },
		neutral			= { 1, 1, 0 },
		pet				= { 0, 1, 0 },
		groupPet		= { 0, 1, 0 },
	},
})

Aloft.Options.args.healthBar =
{
	type = 'group',
	name = L["Health Bar"],
	desc = L["Health Bar options"],
	disabled = function() return not Aloft:IsActive() end,
	args =
	{
		texture =
		{
			type = "text", 
			name = L["Texture"], 
			desc = L["Sets the health bar texture"],
			get = function() return AloftHealthBar.db.profile.texture end,
			set = function(v) AloftHealthBar.db.profile.texture = v AloftHealthBar:UpdateAll() end,
			validate = SML_statusbar,
			order = 1,
		},
		alpha =
		{
			type = 'range',
			name = L["Alpha"],
			desc = L["Sets the health bar alpha"],
			min = 0,
			max = 1.0,
			step = 0.05,
			get = function() return AloftHealthBar.db.profile.alpha end,
			set = function(v) AloftHealthBar.db.profile.alpha = v AloftHealthBar:UpdateAll() end,
			order = 2,
		},
		size =
		{
			type = 'group',
			name = L["Size"],
			desc = L["Adjust health bar's size"],
			args =
			{
				width =
				{
					type = 'range',
					name = L["Width"],
					desc = L["Sets the width of the health bar"],
					min = 50,
					max = 250,
					step = 1,
					get = function() return AloftHealthBar.db.profile.width end,
					set = function(v) AloftHealthBar.db.profile.width = v AloftHealthBar:UpdateSizesAll() end,
					order = 3,
				},
				height =
				{
					type = 'range',
					name = L["Height"],
					desc = L["Sets the height of the health bar"],
					min = 4,
					max = 40,
					step = 1,
					get = function() return AloftHealthBar.db.profile.height end,
					set = function(v) AloftHealthBar.db.profile.height = v AloftHealthBar:UpdateSizesAll() end,
					order = 4,
				},
			}
		},
		position =
		{
			type = 'group',
			name = L["Position"],
			desc = L["Adjust the health bar's position"],
			args =
			{
				offsetX =
				{
					type = 'range',
					name = L["X Offset"],
					desc = L["X offset of the health bar"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftHealthBar.db.profile.offsetX end,
					set = function(v) AloftHealthBar.db.profile.offsetX = v AloftHealthBar:UpdateAll() end
				},
				offsetY =
				{
					type = 'range',
					name = L["Y Offset"],
					desc = L["Y offset of the health bar"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftHealthBar.db.profile.offsetY end,
					set = function(v) AloftHealthBar.db.profile.offsetY = v AloftHealthBar:UpdateAll() end
				},
			},
		},
		colors =
		{
			type = 'group',
			name = L["Colors"],
			desc = L["Customize colors"],
			order = 7,
			args =
			{
				colorByClass =
				{
					type = 'toggle',
					name = L["Color by Class"],
					desc = L["Sets the health bar color of players according to class"],
					get = function() return AloftHealthBar.db.profile.colorByClass end,
					set = function(v) AloftHealthBar.db.profile.colorByClass = v Aloft:DetermineDataSources() AloftHealthBar:UpdateAll() end,
					order = 1
				},
				colorHostileByClass =
				{
					type = 'toggle',
					name = L["Color Hostile by Class"],
					desc = L["Sets the health bar color of hostile players according to class"],
					get = function() return AloftHealthBar.db.profile.colorHostileByClass end,
					set = function(v) AloftHealthBar.db.profile.colorHostileByClass = v Aloft:DetermineDataSources() AloftHealthBar:UpdateAll() end,
					order = 2
				},
				backdrop =
				{
					type = 'color',
					name = L["Backdrop Color"],
					desc = L["Sets the backdrop color of the health bar"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.backdropColor) end,
					set = function(r, g, b, a) AloftHealthBar.db.profile.colors.backdropColor = { r, g, b, a } AloftHealthBar:UpdateAll() end,
					hasAlpha = true,
					order = 3
				},
				friendlyPlayer =
				{
					type = 'color',
					name = L["Friendly Players"],
					desc = L["Sets the health bar color for friendly players"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.friendlyPlayer) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.friendlyPlayer = { r, g, b } AloftHealthBar:UpdateAll() end
				},
				friendlyPet =
				{
					type = 'color',
					name = L["Friendly Pets"],
					desc = L["Sets the health bar color for friendly pets"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.friendlyPet) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.friendlyPet = { r, g, b } Aloft:DetermineDataSources() AloftHealthBar:UpdateAll() end
				},
				friendlyNPC =
				{
					type = 'color',
					name = L["Friendly NPCs"],
					desc = L["Sets the health bar color for friendly NPCs"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.friendlyNPC) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.friendlyNPC = { r, g, b } Aloft:DetermineDataSources() AloftHealthBar:UpdateAll() end
				},
				friendlyBoss =
				{
					type = 'color',
					name = L["Friendly Boss"],
					desc = L["Sets the health bar color for friendly NPC Bosses"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.friendlyBoss) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.friendlyBoss = { r, g, b } AloftHealthBar:UpdateAll() end
				},
				hostilePlayer =
				{
					type = 'color',
					name = L["Hostile Players"],
					desc = L["Sets the health bar color for hostile players"],
					disabled = function() return AloftHealthBar.db.profile.colorHostileByClass end,
					get = function() return unpack(AloftHealthBar.db.profile.colors.hostilePlayer) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.hostilePlayer = { r, g, b } Aloft:DetermineDataSources() AloftHealthBar:UpdateAll() end
				},
				hostilePet  =
				{
					type = 'color',
					name = L["Hostile Pets"],
					desc = L["Sets the health bar color for hostile pets"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.hostilePet) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.hostilePet = { r, g, b } Aloft:DetermineDataSources() AloftHealthBar:UpdateAll() end
				},
				hostileNPC =
				{
					type = 'color',
					name = L["Hostile NPCs"],
					desc = L["Sets the health bar color for hostile units"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.hostileNPC) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.hostileNPC = { r, g, b } Aloft:DetermineDataSources() AloftHealthBar:UpdateAll() end
				},
				hostileBoss =
				{
					type = 'color',
					name = L["Hostile Boss"],
					desc = L["Sets the health bar color for hostile boss units"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.hostileBoss) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.hostileBoss = { r, g, b } AloftHealthBar:UpdateAll() end
				},
				neutral =
				{
					type = 'color',
					name = L["Neutral Units"],
					desc = L["Sets the health bar color for neutral units"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.neutral) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.neutral = { r, g, b } AloftHealthBar:UpdateAll() end
				},
				pet =
				{
					type = 'color',
					name = L["Pet"],
					desc = L["Sets the health bar color for your pet"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.pet) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.pet = { r, g, b } AloftHealthBar:UpdateAll() end
				},
				groupPet =
				{
					type = 'color',
					name = L["Group Pets"],
					desc = L["Sets the health bar color for group pets"],
					get = function() return unpack(AloftHealthBar.db.profile.colors.groupPet) end,
					set = function(r, g, b) AloftHealthBar.db.profile.colors.groupPet = { r, g, b } Aloft:DetermineDataSources() AloftHealthBar:UpdateAll() end
				},
				reset =
				{
					type = "execute",
					name = L["Reset to Defaults"],
					desc = L["Resets all colors to their defaults"],
					order = -1,
					func  = function() 
								local colors = AloftHealthBar.db.profile.colors
								colors.backdropColor	= { 0.25, 0.25, 0.25, 0.5 }
								colors.friendlyPlayer	= { 0, 0, 1 }
								colors.friendlyPet		= { 0, 1, 0 }
								colors.friendlyNPC		= { 0, 1, 0 }
								colors.friendlyBoss		= { 0, 1, 0 }
								colors.hostilePlayer	= { 1, 0, 0 }
								colors.hostilePet		= { 1, 0, 0 }
								colors.hostileNPC		= { 1, 0, 0 }
								colors.hostileBoss		= { 1, 0, 0 }
								colors.neutral			= { 1, 1, 0 }
								colors.pet				= { 0, 1, 0 }
								colors.groupPet			= { 0, 1, 0 }
								Aloft:DetermineDataSources()
								AloftHealthBar:UpdateAll()
							end,
				},
			}
		}
	},
}

-----------------------------------------------------------------------------

function AloftHealthBar:UpdateAll()
	for aloftData in Aloft:IterateNameplates() do
		self:SetupFrame(aloftData)
	end

	for aloftData in Aloft:IterateVisibleNameplates() do
		self:Update(aloftData)
	end

	if self.db.profile.colorByClass then
		self:RegisterEvent("Aloft:OnClassDataChanged", "Update")
	else
		self:CancelEvent("Aloft:OnClassDataChanged")
	end

	if not self:IsArrayEqual(self.db.profile.colors.friendlyPet, self.db.profile.colors.friendlyNPC) or
	   not self:IsArrayEqual(self.db.profile.colors.hostilePet, self.db.profile.colors.hostileNPC) then
		self:RegisterEvent("Aloft:OnIsPetDataChanged", "Update")
	else
		self:CancelEvent("Aloft:OnIsPetDataChanged")
	end
end

function AloftHealthBar:UpdateSizesAll()
	for aloftData in Aloft:IterateNameplates() do
		self:OnNameplateShow(aloftData)
	end

	for aloftData in Aloft:IterateVisibleNameplates() do
		self:Update(aloftData)
	end

	Aloft:GetModule("Frame"):UpdateAll()
	Aloft:GetModule("Overlay"):UpdateAll()
end

-----------------------------------------------------------------------------

function AloftHealthBar:OnEnable()
	self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
	self:RegisterEvent("Aloft:OnNameplateShow", "OnNameplateShow")
	self:RegisterEvent("Aloft:OnHealthBarValueChanged", "Update")
	self:RegisterEvent("Aloft:OnHealthBarColorChanged", "Update")

	self:RegisterEvent("SharedMedia_SetGlobal", function(mediatype, override)
		if mediatype == "statusbar" then
			self:UpdateAll()
		end
	end)
	
	self:RegisterEvent("Aloft:SetAll", function(type, value)
		if self.db.profile[type] then
			self.db.profile[type] = value
			self:UpdateAll()
		end
	end)

	self:UpdateAll()
	self:UpdateSizesAll()
end

local backdropTable = { }

function AloftHealthBar:OnDisable()
	self:UnregisterAllEvents()
	
	backdropTable.bgFile = ""

	for aloftData in Aloft:IterateNameplates() do
		local healthBar = aloftData.healthBar
		
		healthBar:ClearAllPoints()
		healthBar:SetPoint("BOTTOMLEFT", aloftData.nameplateFrame, "BOTTOMLEFT", 4.5, 4.5)
		healthBar:SetWidth(116.5)
		healthBar:SetHeight(10.18)
		healthBar:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
		healthBar:SetStatusBarColor(aloftData.originalHealthBarR, aloftData.originalHealthBarG, aloftData.originalHealthBarB)
		healthBar:SetBackdrop(backdropTable)
		healthBar:SetBackdropColor(0, 0, 0, 0)
		healthBar:SetFrameLevel(0)
	end
end

local dataRequiredList = { }
function AloftHealthBar:RequiresData()
	local profile = self.db.profile

	for i = 1,#dataRequiredList do
		dataRequiredList[i] = nil
	end

	if profile.colorByClass or
	   profile.colorHostileByClass or
	   not self:IsArrayEqual(profile.colors.hostileNPC, profile.colors.hostilePlayer) then
		table.insert(dataRequiredList, "class")
	end

	if not self:IsArrayEqual(profile.colors.friendlyPet, profile.colors.friendlyNPC) or
	   not self:IsArrayEqual(profile.colors.hostilePet, profile.colors.hostileNPC) then
		table.insert(dataRequiredList, "isPet")
	end

	if Aloft:IsDataAvailable("unitid") and not self:IsArrayEqual(profile.colors.friendlyPet, profile.colors.groupPet) then
		table.insert(dataRequiredList, "unitid")
	end

	return unpack(dataRequiredList)
end

function AloftHealthBar:SetupFrame(aloftData)
	local healthBar = aloftData.healthBar	
	local profile = self.db.profile

	healthBar:ClearAllPoints()
	healthBar:SetPoint("CENTER", aloftData.nameplateFrame, "CENTER", profile.offsetX, profile.offsetY)

	local texture = SML:Fetch("statusbar", profile.texture)
	backdropTable.bgFile = texture

	healthBar:SetStatusBarTexture(texture)
	healthBar:SetBackdrop(backdropTable)
	healthBar:SetBackdropColor(unpack(profile.colors.backdropColor))
	healthBar:SetFrameLevel(1)
end

function AloftHealthBar:OnNameplateShow(aloftData)
	local healthBar = aloftData.healthBar
	local profile = self.db.profile

	healthBar:ClearAllPoints()
	healthBar:SetPoint("CENTER", aloftData.nameplateFrame, "CENTER", profile.offsetX, profile.offsetY)
	healthBar:SetWidth(profile.width)
	healthBar:SetHeight(profile.height)

	self:Update(aloftData)	
end

-----------------------------------------------------------------------------

function AloftHealthBar:SetHealthBarColor(healthBar, color)
	healthBar:SetStatusBarColor(color[1], color[2], color[3], self.db.profile.alpha)
end

function AloftHealthBar:ColourFriendlyPlayer(aloftData, healthBar)
	if self.db.profile.colorByClass then
		local class = aloftData.class
		if class then
			local color = RAID_CLASS_COLORS[class]
			if color then
				healthBar:SetStatusBarColor(color.r, color.g, color.b, self.db.profile.alpha)
				return
			end
		end
	end
	self:SetHealthBarColor(healthBar, self.db.profile.colors.friendlyPlayer)
end

function AloftHealthBar:ColourFriendlyNPC(aloftData, healthBar)
	-- Could be player pet, group pet, friendly pet, friendlyBoss, or friendlyNPC
	if aloftData.name == UnitName("pet") then
		self:SetHealthBarColor(healthBar, self.db.profile.colors.pet)
	elseif aloftData.unitid then
		self:SetHealthBarColor(healthBar, self.db.profile.colors.groupPet)
	elseif aloftData.isPet then
		self:SetHealthBarColor(healthBar, self.db.profile.colors.friendlyPet)
	elseif aloftData.isBoss then
		self:SetHealthBarColor(healthBar, self.db.profile.colors.friendlyBoss)
	else
		self:SetHealthBarColor(healthBar, self.db.profile.colors.friendlyNPC)
	end
end

function AloftHealthBar:ColourHostile(aloftData, healthBar)
	-- player, hostile pet, hostile boss, hostileNPC
	if self.db.profile.colorHostileByClass then
		local class = aloftData.class
		if class then
			local color = RAID_CLASS_COLORS[class]
			if color then
				healthBar:SetStatusBarColor(color.r, color.g, color.b, self.db.profile.alpha)
				return
			end
		end
	end

	if aloftData.class then
		self:SetHealthBarColor(healthBar, self.db.profile.colors.hostilePlayer)
	elseif aloftData.isPet then
		self:SetHealthBarColor(healthBar, self.db.profile.colors.hostilePet)
	elseif aloftData.isBoss then
		self:SetHealthBarColor(healthBar, self.db.profile.colors.hostileBoss)
	else
		self:SetHealthBarColor(healthBar, self.db.profile.colors.hostileNPC)
	end
end

function AloftHealthBar:ColourNeutral(aloftData, healthBar)
	self:SetHealthBarColor(healthBar, self.db.profile.colors.neutral)
end

function AloftHealthBar:ColourUnknown(aloftData, healthBar)
	-- Should be here, but if we do, just change the alpha
	local r, g, b = healthBar:GetStatusBarColor()
	healthBar:SetStatusBarColor(r, g, b, self.db.profile.alpha)
end

local healthBarMethods =
{
	["friendlyPlayer"] = AloftHealthBar.ColourFriendlyPlayer,
	["friendlyNPC"] = AloftHealthBar.ColourFriendlyNPC,
	["hostile"] = AloftHealthBar.ColourHostile,
	["neutral"] = AloftHealthBar.ColourNeutral,
	["unknown"] = AloftHealthBar.ColourUnknown,
}

function AloftHealthBar:Update(aloftData)
	local healthBar = aloftData.healthBar

	healthBarMethods[aloftData.type](self, aloftData, healthBar)
end

-----------------------------------------------------------------------------
